"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Intercom = require("intercom-client");
const _ = require("lodash");
const RateLimiter = require("request-rate-limiter");
const companyDataValidation_1 = require("./utils/companyDataValidation");
const parseTools_1 = require("./utils/parseTools");
class IntercomService {
    constructor({ token, MockEffect }) {
        this.intercom = MockEffect ? MockEffect : new Intercom.Client({ token: token });
    }
    tagCompany(params) {
        return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
            try {
                const { intercom } = this;
                const tagToSave = params.tag;
                const actionResult = yield intercom.tags.tag({
                    name: tagToSave,
                    companies: [{ company_id: `${params.company_id}` }],
                });
                const companyInfo = actionResult.body;
                resolve({
                    success: true,
                    data: {
                        internal_id: params.company_id,
                        intercom_id: companyInfo.id,
                        result: companyInfo,
                    },
                });
            }
            catch (error) {
                const fullErr = parseTools_1.jsonParse(error);
                const err = fullErr.message;
                const errorCode = err && err.body && err.body.errors && err.body.errors[0]
                    ? err.body.errors[0].code
                    : 'unknown_error';
                const errorMessage = err && err.body && err.body.errors && err.body.errors[0]
                    ? err.body.errors[0].message
                    : `Error when tagging company with id '${params.company_id}' in Intercom`;
                resolve({
                    success: false,
                    error: {
                        code: errorCode,
                        message: errorMessage,
                        errors: [errorMessage],
                        data: {
                            internal_id: params.company_id,
                        },
                    },
                });
            }
        }));
    }
    tagMultiple(params) {
        return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
            try {
                const { intercom } = this;
                const name = params.name;
                const actionResult = yield intercom.tags.tag({
                    name,
                    companies: params.companies,
                    users: params.users,
                });
                const tag = actionResult.body;
                resolve({
                    success: true,
                    data: {
                        result: tag,
                    },
                });
            }
            catch (error) {
                const fullErr = parseTools_1.jsonParse(error);
                const err = fullErr.message;
                const statusCode = err && err.statusCode ? err.statusCode : -1;
                const errorCode = err && err.body && err.body.errors && err.body.errors[0]
                    ? err.body.errors[0].code
                    : 'unknown_error';
                const errorMessage = err && err.body && err.body.errors && err.body.errors[0]
                    ? err.body.errors[0].message
                    : `Error when creating/updating tags in Intercom.`;
                resolve({
                    success: false,
                    error: {
                        statusCode: statusCode,
                        code: errorCode,
                        message: errorMessage,
                    },
                });
            }
        }));
    }
    createOrUpdateCompany(companyData) {
        return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
            try {
                const { intercom } = this;
                let validated = false;
                const validationResult = companyDataValidation_1.validateDataObject(companyData);
                if (validationResult.validated) {
                    const actionResult = yield intercom.companies.create(companyData);
                    const companyInfo = actionResult.body;
                    resolve({
                        success: true,
                        data: {
                            internal_id: companyData.company_id,
                            intercom_id: companyInfo.id,
                            result: companyInfo,
                        },
                    });
                }
                else {
                    resolve({
                        success: false,
                        error: {
                            code: 'ValidationFailed',
                            message: 'The incoming company object could not be validated',
                            errors: validationResult.errors,
                            data: {
                                internal_id: companyData.company_id,
                            },
                        },
                    });
                }
            }
            catch (error) {
                const fullErr = parseTools_1.jsonParse(error);
                const err = fullErr.message;
                const errorCode = err && err.body && err.body.errors && err.body.errors[0]
                    ? err.body.errors[0].code
                    : 'unknown_error';
                const errorMessage = err && err.body && err.body.errors && err.body.errors[0]
                    ? err.body.errors[0].message
                    : `Error when creating/updating company with id '${companyData.company_id}' in Intercom.`;
                resolve({
                    success: false,
                    error: {
                        code: errorCode,
                        message: errorMessage,
                        errors: [errorMessage],
                        data: {
                            internal_id: companyData.company_id,
                        },
                    },
                });
            }
        }));
    }
    createOrUpdateUser(userData) {
        return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
            try {
                const { intercom } = this;
                const validationResult = companyDataValidation_1.validateDataObject(userData);
                if (validationResult.validated) {
                    const actionResult = yield intercom.users.create(userData);
                    const userInfo = actionResult.body;
                    resolve({
                        success: true,
                        data: {
                            internal_id: userData.user_id,
                            intercom_id: userInfo.id,
                            result: userInfo,
                        },
                    });
                }
                else {
                    resolve({
                        success: false,
                        error: {
                            code: 'validation_failed',
                            message: 'The validation of the incoming user object failed',
                            errors: validationResult.errors,
                            data: {
                                internal_id: userData.user_id,
                            },
                        },
                    });
                }
            }
            catch (error) {
                const fullErr = parseTools_1.jsonParse(error);
                const err = fullErr.message;
                const statusCode = err && err.statusCode ? err.statusCode : -1;
                const errorCode = err && err.body && err.body.errors && err.body.errors[0]
                    ? err.body.errors[0].code
                    : 'unknown_error';
                const errorMessage = err && err.body && err.body.errors && err.body.errors[0]
                    ? err.body.errors[0].message
                    : `Error when creating/updating user with id '${userData.user_id}' in Intercom.`;
                resolve({
                    success: false,
                    error: {
                        statusCode: statusCode,
                        code: errorCode,
                        message: errorMessage,
                        errors: [errorMessage],
                        data: {
                            internal_id: userData.user_id,
                        },
                    },
                });
            }
        }));
    }
    deleteUser(deleteUserParams) {
        return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
            try {
                const { intercom } = this;
                const actionResult = yield intercom.users.delete(deleteUserParams);
                const actionInfo = actionResult.body;
                resolve({
                    success: true,
                    data: {
                        internal_id: deleteUserParams.user_id,
                        intercom_id: actionInfo.id,
                        result: actionInfo,
                    },
                });
            }
            catch (error) {
                const fullErr = parseTools_1.jsonParse(error);
                const err = fullErr.message;
                const errorCode = err && err.body && err.body.errors && err.body.errors[0]
                    ? err.body.errors[0].code
                    : 'unknown_error';
                const errorMessage = err && err.body && err.body.errors && err.body.errors[0]
                    ? err.body.errors[0].message
                    : `Error when deleting user with id '${deleteUserParams.user_id}' in Intercom.`;
                resolve({
                    success: false,
                    error: {
                        code: errorCode,
                        message: errorMessage,
                        errors: [errorMessage],
                        data: {
                            internal_id: deleteUserParams.user_id,
                        },
                    },
                });
            }
        }));
    }
    updateUsersInBulk(users) {
        return new Promise(resolve => {
            if (_.size(users) < 1) {
                resolve({
                    success: false,
                    error: {
                        code: 'NO_DATA',
                        message: 'The size of the incoming array of users where less than 1 so nothing to do here',
                        errors: [
                            'The size of the incoming array of users where less than 1 so nothing to do here',
                        ],
                    },
                });
                return;
            }
            const limiter = new RateLimiter({
                rate: 80,
                interval: 10,
                backoffCode: 429,
                backoffTime: 10,
                maxWaitingTime: 300,
            });
            const errors = [];
            _.forEach(users, user => {
                limiter
                    .request()
                    .then(backoff => {
                    this.createOrUpdateUser({
                        user_id: `${user.user_id}`,
                        name: user.name,
                        email: user.email,
                    }).then(userUpdate => {
                        if (!userUpdate.success) {
                            if (_.isEqual(userUpdate.error.statusCode, 429)) {
                                backoff();
                            }
                            else {
                                errors.push(userUpdate.error);
                            }
                        }
                    });
                })
                    .catch(error => {
                    errors.push(error.message);
                });
            });
            if (_.size(errors) > 0) {
                resolve({
                    success: false,
                    error: {
                        code: 'operation_error',
                        message: 'The bulk operation experienced errors. See the errors array for more information',
                        errors,
                    },
                });
            }
            else {
                resolve({
                    success: true,
                });
            }
        });
    }
}
exports.IntercomService = IntercomService;
//# sourceMappingURL=index.js.map